# -*- coding: utf-8 -*-
from pylab import np, plt, mpl
import control

def get_coeffs_dc_motor_current_regulator(R, L, Bandwidth_Hz):
    Kp = Bandwidth_Hz * 2 * np.pi * L
    Ki = R / L
    # print('DEBUG', Ki, R, L)
    return Kp, Ki

def get_coeffs_dc_motor_SPEED_regulator(J_s, n_pp, KE, delta, currentBandwidth_radPerSec):
    speedKi = currentBandwidth_radPerSec / delta**2
    speedKp = J_s/n_pp / (1.5*n_pp*KE) * delta * speedKi
    return speedKp, speedKi

def 逆上位机速度PI系数转换CODE(iSMC_speedKp, iSMC_speedKiCode, VL_TS, J_s):
    上位机速度KP = iSMC_speedKp / (2*np.pi * J_s)
    iSMC_speedKi = iSMC_speedKiCode/iSMC_speedKp/VL_TS
    上位机速度KI = iSMC_speedKi / (上位机速度KP * np.pi/2) * 1000
    return 上位机速度KP, 上位机速度KI

def 上位机速度PI系数转换CODE(上位机速度KP, 上位机速度KI, VL_TS, J_s):
    # 源代码
        # Kvp_in = 上位机速度KP
        # Kvi_in = 上位机速度KI/1000 * kvp_in * np.pi/2
        # iSMC_speedKp = kvp_in * 2*np.pi * J_s
        # iSMC_speedKi = iSMC_speedKp * kvi_in * VL_TS
    # 简洁点：
    iSMC_speedKp = 上位机速度KP * 2*np.pi * J_s
    iSMC_speedKi = 上位机速度KI/1000 * 上位机速度KP * np.pi/2
    iSMC_speedKiCode = iSMC_speedKi * iSMC_speedKp  * VL_TS
    return iSMC_speedKp, iSMC_speedKi, iSMC_speedKiCode

# current reference to current measurement
def c2c_design(R, L, CLBW_Hz=1000, CL_TS=1/20e3):
    currentKp, currentKi = get_coeffs_dc_motor_current_regulator(R, L, CLBW_Hz)
    currentKiCode = currentKi * currentKp * CL_TS
    if True:
        # 这里打印的用于实验中CCS的debug窗口检查电流环PI系数
        上位机电流KP = CLBW_Hz
        上位机电流KI = 1000
        iSMC_currentKp = 上位机电流KP * L * 2*np.pi
        iSMC_currentKi = 上位机电流KI/1000 * R/L
        iSMC_currentKiCode = iSMC_currentKi * CL_TS * iSMC_currentKp
        print(f'\tiSMC_currentKp={iSMC_currentKp:g}, \
                  iSMC_currentKi={iSMC_currentKi:g}, \
                  iSMC_currentKiCode={iSMC_currentKiCode:g}')
        print(f'\tSimC_currentKp={currentKp:g}, \
                  SimC_currentKi={currentKi:g}, \
                  SimC_currentKiCode={currentKiCode:g}')
        print(f'\t上位机电流KP={上位机电流KP:g}, \
                  上位机电流KI={上位机电流KI:g}')
    Gi_closed = control.tf([1], [L/currentKp, 1]) # current loop zero-pole cancelled already
    currentBandwidth_radPerSec = currentKp/L

    # 注意，我们研究的开环传递函数是以电流给定为输入的，而不是以转速控制误差为输入，这样仿真和实验容易实现一点。
    c2c_tf = Gi_closed

    # fig5 = plt.figure(fignum)
    # plt.title('Designed Current Ref. to Velocity Meas. Transfer Function')
    mag, phase, omega = control.bode_plot(c2c_tf, 2*np.pi*np.logspace(0,4,500), dB=1, Hz=1, deg=1, lw='0.5', label=f'{CLBW_Hz:g} Hz')
    open_cutoff_frequency_HZ = omega[(np.abs(mag-0.0)).argmin()]/2/np.pi
    # print('\tCut-off frequency (without speed PI regulator):', open_cutoff_frequency_HZ, 'Hz')
    return  (currentKp, currentKi), \
            (上位机电流KP, 上位机电流KI), \
            (mag, phase, omega)

# current reference to velocity measaurement (this is not velocity open loop, because speed PI is not considered)
def c2v_design(R, L, n_pp, J_s, KE, B=0, CLBW_Hz=1000, CL_TS=1/20e3, fignum=5):

    currentKp, currentKi = get_coeffs_dc_motor_current_regulator(R, L, CLBW_Hz)
    currentKiCode = currentKi * currentKp * CL_TS
    if True:
        # 这里打印的用于实验中CCS的debug窗口检查电流环PI系数
        上位机电流KP = CLBW_Hz
        上位机电流KI = 1000
        iSMC_currentKp = 上位机电流KP * L * 2*np.pi
        iSMC_currentKi = 上位机电流KI/1000 * R/L
        iSMC_currentKiCode = iSMC_currentKi * CL_TS * iSMC_currentKp
        print(f'\tiSMC_currentKp={iSMC_currentKp:g}, \
                  iSMC_currentKi={iSMC_currentKi:g}, \
                  iSMC_currentKiCode={iSMC_currentKiCode:g}')
        print(f'\tSimC_currentKp={currentKp:g}, \
                  SimC_currentKi={currentKi:g}, \
                  SimC_currentKiCode={currentKiCode:g}')
        print(f'\t上位机电流KP={上位机电流KP:g}, \
                  上位机电流KI={上位机电流KI:g}')
    Gi_closed = control.tf([1], [L/currentKp, 1]) # current loop zero-pole cancelled already
    currentBandwidth_radPerSec = currentKp/L

    KT = 1.5*n_pp*KE
    dc_motor_motion = control.tf([KT], [J_s/n_pp, B]) # [Apk] to [elec.rad/s]
    print(dc_motor_motion)
    # quit()

    # 注意，我们研究的开环传递函数是以电流给定为输入的，而不是以转速控制误差为输入，这样仿真和实验容易实现一点。
    # Gw_open = dc_motor_motion * Gi_closed * speedPI
    c2v_tf = dc_motor_motion * Gi_closed

    # fig5 = plt.figure(fignum)
    # plt.title('Designed Current Ref. to Velocity Meas. Transfer Function')
    mag, phase, omega = control.bode_plot(c2v_tf, 2*np.pi*np.logspace(0,4,500), dB=1, Hz=1, deg=1, lw='0.5', label=f'{CLBW_Hz:g} Hz')
    open_cutoff_frequency_HZ = omega[(np.abs(mag-0.0)).argmin()]/2/np.pi
    # print('\tCut-off frequency (without speed PI regulator):', open_cutoff_frequency_HZ, 'Hz')
    return  (currentKp, currentKi), \
            (上位机电流KP, 上位机电流KI), \
            (mag, phase, omega)

# velocity reference to velocity measaurement
def iterate_for_desired_bandwidth( delta, desired_VLBW_Hz, motor_dict, CLBW_Hz_initial=1000, CLBW_Hz_stepSize=100):

    # print('DEBUG tuner.py', motor_dict)

    R          = motor_dict['Rs']
    L          = motor_dict['Ls']
    J_s        = motor_dict['J_s']
    JLoadRatio = motor_dict['JLoadRatio']
    n_pp       = motor_dict['n_pp']
    KE         = motor_dict['KE']
    CL_TS      = motor_dict['CL_TS']
    VL_TS      = motor_dict['VL_TS']
    J_total    = J_s*(1+JLoadRatio) 

    CLBW_Hz = CLBW_Hz_initial  #100 # Hz (initial)
    VLBW_Hz = -10  # Hz (initial)
    count = 0

    while True:
        count += 1
        if count>20:
            msg = f'Loop count 20 is reached. Step size is {CLBW_Hz_stepSize} Hz.'
            print(msg)
            # raise Exception()
            break

        # Current loop (Tune its bandwidth to support required speed response)
        if abs(VLBW_Hz - desired_VLBW_Hz)<=10: # Hz
            break
        else:
            if VLBW_Hz > desired_VLBW_Hz:
                CLBW_Hz -= CLBW_Hz_stepSize # Hz
                if CLBW_Hz<=0:
                    raise Exception(f'Negative CLBW_Hz. Maybe change the step size of "CLBW_Hz" ({CLBW_Hz_stepSize} Hz) and try again.')
                    break
            else:
                CLBW_Hz += CLBW_Hz_stepSize # Hz
        # print(f'CLBW_Hz = {CLBW_Hz}')

        currentKp, currentKi = get_coeffs_dc_motor_current_regulator(R, L, CLBW_Hz)
        currentKiCode = currentKi * currentKp * CL_TS
        if True:
            # 这里打印的用于实验中CCS的debug窗口检查电流环PI系数
            上位机电流KP = CLBW_Hz
            上位机电流KI = 1000
            iSMC_currentKp = 上位机电流KP * L * 2*np.pi
            iSMC_currentKi = 上位机电流KI/1000 * R/L
            iSMC_currentKiCode = iSMC_currentKi * CL_TS * iSMC_currentKp
            # print(f'\tiSMC_currentKp={iSMC_currentKp:g}, \
            #           iSMC_currentKi={iSMC_currentKi:g}, \
            #           iSMC_currentKiCode={iSMC_currentKiCode:g}')
            # print(f'\tSimC_currentKp={currentKp:g}, \
            #           SimC_currentKi={currentKi:g}, \
            #           SimC_currentKiCode={currentKiCode:g}')
            # print(f'\t上位机电流KP={上位机电流KP:g}, \
            #           上位机电流KI={上位机电流KI:g}')
        Gi_closed = control.tf([1], [L/currentKp, 1]) # current loop zero-pole cancelled already
        currentBandwidth_radPerSec = currentKp/L

        # Speed loop
        KT = 1.5*n_pp*KE
        dc_motor_motion = control.tf([KT*n_pp/J_total], [1, 0])
        speedKp, speedKi = get_coeffs_dc_motor_SPEED_regulator(J_total, n_pp, KE, delta, currentBandwidth_radPerSec)
        speedKiCode = speedKi * speedKp * VL_TS
        if True:
            # 这里打印的用于实验中TI的debug窗口检查系数

            上位机速度KP, 上位机速度KI = 逆上位机速度PI系数转换CODE(speedKp, speedKiCode, VL_TS, J_total)

            iSMC_speedKp, iSMC_speedKi, iSMC_speedKiCode = 上位机速度PI系数转换CODE(上位机速度KP, 上位机速度KI, VL_TS, J_total)

            # print(f'\tiSMC_speedKp={iSMC_speedKp:g}, \
            #           iSMC_speedKi={iSMC_speedKi:g}, \
            #           iSMC_speedKiCode={iSMC_speedKiCode:g}')
            # print(f'\tSimC_speedKp={speedKp:g}, \
            #           SimC_speedKi={speedKi:g}, \
            #           SimC_speedKiCode={speedKiCode:g}')
            # print(f'\t上位机速度KP={上位机速度KP:g}, \
            #           上位机速度KI={上位机速度KI:g}')
        # 下面打印的用于仿真
        # print(f'\tspeedKp = {speedKp:g}', f'speedKi = {speedKi:g}', \
        #       f'wzero = {speedKi/2/np.pi:g} Hz', \
        #       f'cutoff = {delta*speedKi/2/np.pi:g} Hz', \
        #       f'ipole = {currentKp/L/2/np.pi:g} Hz', sep=' | ')

        speedPI = control.tf([speedKp, speedKp*speedKi], [1, 0])
        Gw_open = dc_motor_motion * Gi_closed * speedPI

        # C2C
        c2c_tf = Gi_closed
        mag, phase, omega = control.bode_plot(c2c_tf, 2*np.pi*np.logspace(0,4,500), dB=1, Hz=1, deg=1, lw='0.5', label=f'{CLBW_Hz:g} Hz')
        CLBW_Hz = omega[(np.abs(mag-0.707)).argmin()]/2/np.pi
        C2C_designedMagPhaseOmega = mag, phase, omega

        # C2V
        c2v_tf = dc_motor_motion * Gi_closed
        mag, phase, omega = control.bode_plot(c2v_tf, 2*np.pi*np.logspace(0,4,500), dB=1, Hz=1, deg=1, lw='0.5', label=f'{CLBW_Hz:g} Hz')
        open_cutoff_frequency_HZ = omega[(np.abs(mag-0.0)).argmin()]/2/np.pi
        C2V_designedMagPhaseOmega = mag, phase, omega

        # V2V
        Gw_closed = Gw_open / (1+Gw_open)
        mag, phase, omega = control.bode_plot(Gw_closed, 2*np.pi*np.logspace(0,4,500), dB=1, Hz=1, deg=1, lw='0.5', label=f'{delta:g}')
        VLBW_Hz = omega[(np.abs(mag-0.707)).argmin()]/2/np.pi
        V2V_designedMagPhaseOmega = mag, phase, omega
        # print(Gw_closed)
        # print('\tSpeed loop bandwidth:', VLBW_Hz, 'Hz')
    return  (currentKp, currentKi), \
            (speedKp, speedKi), \
            (上位机电流KP, 上位机电流KI), \
            (上位机速度KP, 上位机速度KI), \
            (C2C_designedMagPhaseOmega, C2V_designedMagPhaseOmega, V2V_designedMagPhaseOmega), \
            (CLBW_Hz, VLBW_Hz, open_cutoff_frequency_HZ)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'page_messages.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

import resources_rc

class Ui_chat_page(object):
    def setupUi(self, chat_page):
        if not chat_page.objectName():
            chat_page.setObjectName(u"chat_page")
        chat_page.resize(880, 616)
        chat_page.setStyleSheet(u"QWidget { color: rgb(165, 165, 165); }\n"
"#chat_page{\n"
"	background-color: rgb(0, 0, 0);	\n"
"}\n"
"/* TOP */\n"
"#top {\n"
"	background-color: rgb(30, 32, 33);\n"
"	border-radius: 10px;\n"
"}\n"
"#user_name { \n"
"	color: rgb(179, 179, 179);\n"
"	font: 600 12pt \"Segoe UI\";\n"
"}\n"
"#user_image {\n"
"	border: 1px solid rgb(30, 32, 33);\n"
"	background-color: rgb(47, 48, 50);\n"
"	border-radius: 20px;\n"
"}\n"
"#top QPushButton {\n"
"	background-color: rgb(47, 48, 50);\n"
"	border-radius: 20px;\n"
"	background-repeat: no-repeat;\n"
"	background-position: center;\n"
"}\n"
"#top QPushButton:hover {\n"
"	background-color: rgb(61, 62, 65);\n"
"}\n"
"#top QPushButton:pressed {\n"
"	background-color: rgb(16, 17, 18);\n"
"}\n"
"#btn_attachment_top {	\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_attachment.svg);\n"
"}\n"
"#btn_more_top {	\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_more_options.svg);\n"
"}\n"
"/* BOTTOM */\n"
"#bottom QPushButton {\n"
"	background-color: rgb(47, 4"
                        "8, 50);\n"
"	border-radius: 20px;\n"
"	background-repeat: no-repeat;\n"
"	background-position: center;\n"
"}\n"
"#bottom QPushButton:hover {\n"
"	background-color: rgb(61, 62, 65);\n"
"}\n"
"#bottom QPushButton:pressed {\n"
"	background-color: rgb(16, 17, 18);\n"
"}\n"
"#send_message_frame { \n"
"	background-color: rgb(47, 48, 50);\n"
"	border-radius: 20px;\n"
"}\n"
"#send_message_frame QPushButton {\n"
"	background-color: rgb(76, 77, 80);\n"
"	border-radius: 15px;\n"
"	background-repeat: no-repeat;\n"
"	background-position: center;\n"
"}\n"
"#send_message_frame QPushButton:hover {\n"
"	background-color: rgb(81, 82, 86);\n"
"}\n"
"#send_message_frame QPushButton:pressed {\n"
"	background-color: rgb(16, 17, 18);\n"
"}\n"
"#line_edit_message {\n"
"	background-color: transparent;\n"
"	selection-color: rgb(255, 255, 255);\n"
"	selection-background-color: rgb(149, 199, 0);\n"
"	border: none;\n"
"	padding-left: 15px;\n"
"	padding-right: 15px;\n"
"	background-repeat: none;\n"
"	background-position: left center;\n"
"	"
                        "font: 10pt \"Segoe UI\";\n"
"	color: rgb(94, 96, 100);\n"
"}\n"
"#line_edit_message:focus {\n"
"	color: rgb(165, 165, 165);\n"
"}\n"
"#btn_emoticon{\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_emoticons.svg);\n"
"}\n"
"#btn_send_message{	\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_send.svg);\n"
"}\n"
"#btn_attachment_bottom{	\n"
"	\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_more_options.svg);\n"
"}")
        self.verticalLayout = QVBoxLayout(chat_page)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.top = QFrame(chat_page)
        self.top.setObjectName(u"top")
        self.top.setMinimumSize(QSize(0, 60))
        self.top.setMaximumSize(QSize(16777215, 60))
        self.top.setFrameShape(QFrame.NoFrame)
        self.top.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.top)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.user_image = QLabel(self.top)
        self.user_image.setObjectName(u"user_image")
        self.user_image.setMinimumSize(QSize(40, 40))
        self.user_image.setMaximumSize(QSize(40, 40))
        self.user_image.setAlignment(Qt.AlignCenter)

        self.horizontalLayout.addWidget(self.user_image)

        self.user_information_frame = QFrame(self.top)
        self.user_information_frame.setObjectName(u"user_information_frame")
        self.user_information_frame.setFrameShape(QFrame.NoFrame)
        self.user_information_frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.user_information_frame)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(5, 5, 5, 5)
        self.user_name = QLabel(self.user_information_frame)
        self.user_name.setObjectName(u"user_name")
        self.user_name.setMinimumSize(QSize(0, 22))
        self.user_name.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.verticalLayout_2.addWidget(self.user_name)

        self.user_description = QLabel(self.user_information_frame)
        self.user_description.setObjectName(u"user_description")
        self.user_description.setStyleSheet(u"background: transparent;")

        self.verticalLayout_2.addWidget(self.user_description)


        self.horizontalLayout.addWidget(self.user_information_frame)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.last_time_connected = QLabel(self.top)
        self.last_time_connected.setObjectName(u"last_time_connected")

        self.horizontalLayout.addWidget(self.last_time_connected)

        self.btn_attachment_top = QPushButton(self.top)
        self.btn_attachment_top.setObjectName(u"btn_attachment_top")
        self.btn_attachment_top.setMinimumSize(QSize(40, 40))
        self.btn_attachment_top.setMaximumSize(QSize(40, 40))
        self.btn_attachment_top.setCursor(QCursor(Qt.PointingHandCursor))

        self.horizontalLayout.addWidget(self.btn_attachment_top)

        self.btn_more_top = QPushButton(self.top)
        self.btn_more_top.setObjectName(u"btn_more_top")
        self.btn_more_top.setMinimumSize(QSize(40, 40))
        self.btn_more_top.setMaximumSize(QSize(40, 40))
        self.btn_more_top.setCursor(QCursor(Qt.PointingHandCursor))

        self.horizontalLayout.addWidget(self.btn_more_top)


        self.verticalLayout.addWidget(self.top)

        self.chat_messages = QScrollArea(chat_page)
        self.chat_messages.setObjectName(u"chat_messages")
        self.chat_messages.setStyleSheet(u"background: transparent")
        self.chat_messages.setFrameShape(QFrame.NoFrame)
        self.chat_messages.setFrameShadow(QFrame.Raised)
        self.chat_messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.chat_messages.setWidgetResizable(True)
        self.chat_messages.setAlignment(Qt.AlignBottom|Qt.AlignLeading|Qt.AlignLeft)
        self.messages_widget = QWidget()
        self.messages_widget.setObjectName(u"messages_widget")
        self.messages_widget.setGeometry(QRect(0, 0, 862, 486))
        self.messages_widget.setStyleSheet(u"background: transparent")
        self.chat_layout = QVBoxLayout(self.messages_widget)
        self.chat_layout.setSpacing(0)
        self.chat_layout.setObjectName(u"chat_layout")
        self.chat_layout.setContentsMargins(0, 0, 0, 0)
        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.chat_layout.addItem(self.verticalSpacer)

        self.messages_frame = QFrame(self.messages_widget)
        self.messages_frame.setObjectName(u"messages_frame")
        self.messages_frame.setFrameShape(QFrame.NoFrame)
        self.messages_frame.setFrameShadow(QFrame.Raised)
        self.chat_messages_layout = QVBoxLayout(self.messages_frame)
        self.chat_messages_layout.setSpacing(0)
        self.chat_messages_layout.setObjectName(u"chat_messages_layout")
        self.chat_messages_layout.setContentsMargins(0, 0, 0, 0)

        self.chat_layout.addWidget(self.messages_frame)

        self.chat_messages.setWidget(self.messages_widget)

        self.verticalLayout.addWidget(self.chat_messages)

        self.bottom = QFrame(chat_page)
        self.bottom.setObjectName(u"bottom")
        self.bottom.setMinimumSize(QSize(0, 40))
        self.bottom.setMaximumSize(QSize(16777215, 40))
        self.bottom.setFrameShape(QFrame.NoFrame)
        self.bottom.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.bottom)
        self.horizontalLayout_2.setSpacing(10)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.send_message_frame = QFrame(self.bottom)
        self.send_message_frame.setObjectName(u"send_message_frame")
        self.send_message_frame.setFrameShape(QFrame.StyledPanel)
        self.send_message_frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_3 = QHBoxLayout(self.send_message_frame)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(5, 0, 5, 0)
        self.btn_emoticon = QPushButton(self.send_message_frame)
        self.btn_emoticon.setObjectName(u"btn_emoticon")
        self.btn_emoticon.setMinimumSize(QSize(30, 30))
        self.btn_emoticon.setMaximumSize(QSize(30, 30))
        self.btn_emoticon.setCursor(QCursor(Qt.PointingHandCursor))

        self.horizontalLayout_3.addWidget(self.btn_emoticon)

        self.line_edit_message = QLineEdit(self.send_message_frame)
        self.line_edit_message.setObjectName(u"line_edit_message")
        self.line_edit_message.setMinimumSize(QSize(0, 40))

        self.horizontalLayout_3.addWidget(self.line_edit_message)

        self.btn_send_message = QPushButton(self.send_message_frame)
        self.btn_send_message.setObjectName(u"btn_send_message")
        self.btn_send_message.setMinimumSize(QSize(30, 30))
        self.btn_send_message.setMaximumSize(QSize(30, 30))
        self.btn_send_message.setCursor(QCursor(Qt.PointingHandCursor))

        self.horizontalLayout_3.addWidget(self.btn_send_message)


        self.horizontalLayout_2.addWidget(self.send_message_frame)

        self.btn_attachment_bottom = QPushButton(self.bottom)
        self.btn_attachment_bottom.setObjectName(u"btn_attachment_bottom")
        self.btn_attachment_bottom.setMinimumSize(QSize(40, 40))
        self.btn_attachment_bottom.setMaximumSize(QSize(40, 40))
        self.btn_attachment_bottom.setCursor(QCursor(Qt.PointingHandCursor))

        self.horizontalLayout_2.addWidget(self.btn_attachment_bottom)


        self.verticalLayout.addWidget(self.bottom)


        self.retranslateUi(chat_page)

        QMetaObject.connectSlotsByName(chat_page)
    # setupUi

    def retranslateUi(self, chat_page):
        chat_page.setWindowTitle(QCoreApplication.translate("chat_page", u"Form", None))
        self.user_image.setText("")
        self.user_name.setText(QCoreApplication.translate("chat_page", u"User name", None))
        self.user_description.setText(QCoreApplication.translate("chat_page", u"User description", None))
        self.last_time_connected.setText(QCoreApplication.translate("chat_page", u"connected last time 24h ago", None))
        self.btn_attachment_top.setText("")
        self.btn_more_top.setText("")
        self.btn_emoticon.setText("")
        self.line_edit_message.setPlaceholderText(QCoreApplication.translate("chat_page", u"Message #user", None))
        self.btn_send_message.setText("")
        self.btn_attachment_bottom.setText("")
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'login.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

import resources_rc

class Ui_Login(object):
    def setupUi(self, Login):
        if not Login.objectName():
            Login.setObjectName(u"Login")
        Login.resize(300, 420)
        Login.setMinimumSize(QSize(300, 420))
        Login.setMaximumSize(QSize(300, 420))
        Login.setStyleSheet(u"#bg {\n"
"	background-color: rgb(0, 0, 0);\n"
"	border-radius: 10px;\n"
"}\n"
"QLabel {\n"
"	color:  rgb(121, 121, 121);\n"
"	padding-left: 10px;\n"
"	padding-top: 20px;\n"
"}\n"
".QLineEdit {\n"
"	border: 3px solid rgb(47, 48, 50);\n"
"	border-radius: 15px;\n"
"	background-color: rgb(47, 48, 50);\n"
"	color: rgb(121, 121, 121);\n"
"	padding-left: 10px;\n"
"	padding-right: 10px;\n"
"	background-repeat: none;\n"
"	background-position: left center;\n"
"}\n"
".QLineEdit:hover {\n"
"	color: rgb(230, 230, 230);\n"
"	border: 3px solid rgb(62, 63, 66);\n"
"}\n"
".QLineEdit:focus {\n"
"	color: rgb(230, 230, 230);\n"
"	border: 3px solid rgb(189, 255, 0);\n"
"	background-color: rgb(14, 14, 15);\n"
"}")
        self.centralwidget = QWidget(Login)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(10, 10, 10, 10)
        self.bg = QFrame(self.centralwidget)
        self.bg.setObjectName(u"bg")
        self.bg.setFrameShape(QFrame.NoFrame)
        self.bg.setFrameShadow(QFrame.Raised)
        self.frame_widgets = QFrame(self.bg)
        self.frame_widgets.setObjectName(u"frame_widgets")
        self.frame_widgets.setGeometry(QRect(0, 70, 280, 720))
        self.frame_widgets.setMinimumSize(QSize(280, 720))
        self.frame_widgets.setFrameShape(QFrame.NoFrame)
        self.frame_widgets.setFrameShadow(QFrame.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.frame_widgets)
        self.verticalLayout_2.setSpacing(5)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(20, 10, 20, 10)
        self.preloader = QFrame(self.frame_widgets)
        self.preloader.setObjectName(u"preloader")
        self.preloader.setMinimumSize(QSize(240, 240))
        self.preloader.setMaximumSize(QSize(260, 260))
        self.preloader.setFrameShape(QFrame.NoFrame)
        self.preloader.setFrameShadow(QFrame.Raised)

        self.verticalLayout_2.addWidget(self.preloader)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.logo = QFrame(self.frame_widgets)
        self.logo.setObjectName(u"logo")
        self.logo.setMinimumSize(QSize(0, 260))
        self.logo.setStyleSheet(u"#logo {\n"
"	border-radius: 10px;\n"
"	background-image: url(:/images_svg/images/images_svg/logo_home.svg);\n"
"	background-position: center;\n"
"	background-repeat: no-repeat;\n"
"}")
        self.logo.setFrameShape(QFrame.NoFrame)
        self.logo.setFrameShadow(QFrame.Raised)

        self.verticalLayout_2.addWidget(self.logo)

        self.user_description = QLabel(self.frame_widgets)
        self.user_description.setObjectName(u"user_description")
        self.user_description.setStyleSheet(u"background: transparent;")

        self.verticalLayout_2.addWidget(self.user_description)

        self.username = QLineEdit(self.frame_widgets)
        self.username.setObjectName(u"username")
        self.username.setMinimumSize(QSize(0, 30))
        self.username.setMaximumSize(QSize(16777215, 40))

        self.verticalLayout_2.addWidget(self.username)

        self.password = QLineEdit(self.frame_widgets)
        self.password.setObjectName(u"password")
        self.password.setMinimumSize(QSize(0, 30))
        self.password.setMaximumSize(QSize(16777215, 40))
        self.password.setEchoMode(QLineEdit.Password)

        self.verticalLayout_2.addWidget(self.password)


        self.verticalLayout.addWidget(self.bg)

        Login.setCentralWidget(self.centralwidget)

        self.retranslateUi(Login)

        QMetaObject.connectSlotsByName(Login)
    # setupUi

    def retranslateUi(self, Login):
        Login.setWindowTitle(QCoreApplication.translate("Login", u"Login. PyBlackBOX", None))
        self.user_description.setText(QCoreApplication.translate("Login", u"Login (pass: 123456):", None))
        self.username.setPlaceholderText(QCoreApplication.translate("Login", u"Username", None))
        self.password.setPlaceholderText(QCoreApplication.translate("Login", u"Password", None))
    # retranslateUi




from jsons import ACMInfo
from acmdesignv2 import tuner
from acmdesignv2 import simulator 
from acmdesignv2 import analyzer

# import os 
from pylab import np, plt, mpl
import control
mpl.rcParams['figure.dpi'] = 120
plt.style.use('ggplot')

# import Experiment


if __name__ == '__main__':

    mj = ACMInfo.MotorJson().d
    motor = mj["SEW400W (SF60B04030C2004)"]["基本参数"]

    n_pp = motor["极对数 [1]"]
    R    = motor["电机线电阻 [Ohm]"]/2
    L    = motor["电机D轴线电感 [mH]"]/2*1e-3
    KE   = motor["转矩常数 [Nm/Arms]"] / 1.5 / n_pp * 1.414
    J_s  = motor["转动惯量 [kg.cm^2]"]*1e-4
    IN   = motor["额定电流 [Arms]"]
    PW   = motor["额定功率 [Watt]"]
    RPM  = motor["额定转速 [rpm]"]

    motor_dict = dict()

    motor_dict['DOWN_SAMPLE'] = 1

    motor_dict['n_pp'] = n_pp
    motor_dict['Rs'] = R
    motor_dict['Ld'] = L
    motor_dict['Lq'] = L
    motor_dict['KE'] = KE
    motor_dict['J_s'] = J_s
    motor_dict['IN'] = IN
    motor_dict['PW'] = PW
    motor_dict['RPM'] = RPM

    CL_TS = 1/20e3
    VL_TS = 4*CL_TS # if modified, need to change SPEED_LOOP_CEILING in ACMConfig.h as well
    motor_dict['CL_TS'] = CL_TS
    motor_dict['VL_TS'] = VL_TS

    motor_dict['JLoadRatio'] = JLoadRatio = 0.16 # 0.16 # 3 [%]
    motor_dict['Tload'] = TLoad = 0.0 # 0.05 # [Nm]
    motor_dict['ViscousCoeff'] = B = 0.7e-4

if __name__ == '__main__':
# for JLoadRatio in [0.16, 3, 10, 30]:
    motor_dict['JLoadRatio'] = JLoadRatio

    # Iterate for a preset speed loop bandwidth based on your delta value

    # Pick your favorite step repsonse by "shape" via damping factor \delta
    # for delta in [2, 4, 6.5, 8, 10, 15, 20]:
    # for delta in [2, 6.5,  15]:
    for delta in [6.5]:
    # for delta in [15]:

        # Specify your desired speed closed-loop bandwidth
        # desired_BW_velocity_HZ = 223
        desired_BW_velocity_HZ = 100

        currentPI, speedPI, 上位机电流PI, 上位机速度PI, MagPhaseOmega, BW_in_Hz \
            = tuner.iterate_for_desired_bandwidth(delta, desired_BW_velocity_HZ, motor_dict)
        currentKp, currentKi = currentPI
        speedKp, speedKi = speedPI
        上位机电流KP, 上位机电流KI = 上位机电流PI
        上位机速度KP, 上位机速度KI = 上位机速度PI
        print(f'\n\n\
            #define CURRENT_KP {currentKp:g}\n\
            #define CURRENT_KI {currentKi:g}\n\
            #define CURRENT_KI_CODE (CURRENT_KI*CURRENT_KP*CL_TS)\n\
            #define SPEED_KP {speedKp:g}\n\
            #define SPEED_KI {speedKi:g}\n\
            #define SPEED_KI_CODE (SPEED_KI*SPEED_KP*VL_TS)\n')

        fig5 = plt.figure(5)
        fig5.axes[0].set_ylim([-3, 10]) # -3dB
        fig5.axes[1].set_ylim([-90, 0]) # 90 deg
        print('------------end of tuner\n\n\n')



        motor_dict['data_fname_prefix'] = 'demo-closedLoop'

        max_freq = 2*desired_BW_velocity_HZ
        init_freq = 2

        # change to false to save time
        if True:
            ad = simulator.acm_designer(r'D:\DrH\Codes\acmsimcv5') # os.path.dirname(os.path.realpath(__file__))
            ad.update_ACMConfig_evaluate_PI_coefficients(currentPI, speedPI, 上位机电流PI, 上位机速度PI, 
                                                         max_freq=max_freq, init_freq=init_freq,
                                                         motor_dict=motor_dict)
            data_fname = ad.compile_c_and_run()
            # plt.figure()
            data_file_name = ad.plot_PI_coefficients()
            print('------------end of simulator\n\n\n')
            # plt.show()

        # 1. Ploe simualted Bode plot
        dot_dat_file_dir = r'D:/DrH/Codes/acmsimcv5/dat/' + data_fname

        sweepFreq_dict = dict()
        sweepFreq_dict["max_freq"] = max_freq
        sweepFreq_dict["init_freq"] = init_freq
        sweepFreq_dict["SWEEP_FREQ_C2V"] = False
        sweepFreq_dict["SWEEP_FREQ_C2C"] = False

        print(motor_dict)
        print(sweepFreq_dict)
        dB, Deg, Freq, max_freq = analyzer.analyze(dot_dat_file_dir, motor_dict, sweepFreq_dict)
        # for el in (dB, Deg, Freq, max_freq):
        #     print(el)

        plt.figure(4, figsize=(20,8))
        plt.plot(Freq, dB, '--.', label=data_fname)

        index, value = analyzer.find_nearest(dB, -3) # find -3dB freq
        VLBW = Freq[index]
        plt.text(VLBW, -5, f'{VLBW:.0f} Hz', color='red', fontsize=20)
        plt.plot([0,max_freq], [-3, -3], 'k--')
        plt.ylabel('Velocity Closed-loop transfer function amplitude [dB]')

        plt.xscale('log')
        plt.xlabel('Frequency [Hz]')
        plt.legend()


        # 2. Plot designed Bode plot
        # plt.figure(4, figsize=(20,8))
        mag, phase, omega = MagPhaseOmega
        index_max_freq = sum(omega/(2*np.pi) < max_freq)
        plt.plot((omega/(2*np.pi))[:index_max_freq], 20*np.log10(mag[:index_max_freq]), '-.', label=f'designed:$\\delta={delta}$')

        # 3. Plot measured Bode plot
        # fname = r'D:\ISMC\SweepFreq\Jupyter\VLBW-Data/' + 'BiasSine500rpm' + data_file_name[data_file_name.find(data_fname_prefix)+len(data_fname_prefix)+len('-CLOSED-@'):-4]+'.txt'
        # try:
        #     CL_VL_TF, list_phase_difference, list_qep_max_frequency, max_freq = Experiment.analyze_experimental_measurements(fname)
        # except FileNotFoundError as e:
        #     raise e
        #     print(str(e))
        #     pass
        # except Exception as e:
        #     raise e
        # finally:
        #     index, value = Experiment.find_nearest(CL_VL_TF, -3) # find -3dB freq
        #     VLBW = list_qep_max_frequency[index]
        #     plt.text(VLBW, -5, f'{VLBW:.0f} Hz', color='purple', fontsize=20)
        #     plt.plot(list_qep_max_frequency, CL_VL_TF, '--.', label=fname)

        plt.legend()
        # break


        # 潜在误差来源：DFT提取的幅值是有损失的！

if __name__ == '!__main__':
    analyzer.main(ad, data_fname_prefix=data_fname_prefix)
    print('------------end of analyzer\n\n\n')

if __name__ == '__main__':
    plt.show()



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT
# ///////////////////////////////////////////////////////////////
# Packages
# from app.packages.pyside_or_pyqt import *
# from PySide2.QtWidgets import *
# from PySide2.QtCore import Signal
from app_modules import * # Settings

# Modules
# import app.modules.ui_functions.functions as ui_functions
# from app.modules.app_settings.settings import *
import os

# TOOLTIP / LABEL StyleSheet
style_tooltip = """ 
QLabel {		
	background-color: #0b0b0c;	
	color: rgb(230, 230, 230);
	padding-left: 10px;
	padding-right: 10px;
	border-radius: 17px;
    border: 1px solid #2f3032;
    border-left: 3px solid #bdff00;
    font: 800 9pt "Segoe UI";
}
"""

# CUSTOM LEFT MENU
class LeftMenuButton(QWidget):
    # SIGNALS
    clicked = Signal()
    released = Signal()

    def __init__(self, parent, name, icon, tooltip):
        QWidget.__init__(self)
        # APP PATH
        app_path = os.path.dirname(__file__) # os.path.abspath(os.getcwd())
        icon_path = os.path.join(app_path, icon)

        # GET SETTINGS
        settings = Settings()
        self.settings = settings.items

        # DEFAULT PARAMETERS
        self.width = 40
        self.height = 40
        self.pos_x = 0
        self.pos_y = 0
        self.border_radius = 10
        self.parent = parent
        self.setGeometry(0, 0, self.width, self.height)
        self.setMinimumSize(self.width, self.height)
        self.setCursor(Qt.PointingHandCursor)
        self.setObjectName(name)
        
        # BG COLORS
        self.color_default = QColor(self.settings["left_menu"]["color"])
        self.color_hover = QColor(self.settings["left_menu"]["color_hover"])
        self.color_pressed = QColor(self.settings["left_menu"]["color_pressed"])
        self._set_color = self.color_default

        # ICON
        self.icon_color = QColor(0xE6E6E6)
        self.icon_color_pressed = QColor(0x151617)
        self._set_icon_path = icon_path        
        self._set_icon_color = self.icon_color

        # TOOLTIP
        self.tooltip_text = tooltip
        self.tooltip = _ToolTip(parent, tooltip)
        self.tooltip.hide()

    # PAINT EVENT
    # Responsible for painting the button, as well as the icon
    def paintEvent(self, event):
        # PAINTER
        paint = QPainter()
        paint.begin(self)
        paint.setRenderHint(QPainter.RenderHint.Antialiasing)
        
        # BRUSH
        brush = QBrush(self._set_color)

        # CREATE RECTANGLE
        rect = QRect(0, 0, self.width, self.height)
        paint.setPen(Qt.NoPen)
        paint.setBrush(brush)
        paint.drawRoundedRect(rect, self.border_radius, self.border_radius)

        # DRAW ICONS
        self.icon_paint(paint, self._set_icon_path, rect)

        # END PAINTER
        paint.end()

    # DRAW ICON WITH COLORS
    def icon_paint(self, qp, image, rect):
        icon = QPixmap(image)
        painter = QPainter(icon)
        painter.setCompositionMode(QPainter.CompositionMode_SourceIn)
        painter.fillRect(icon.rect(), self._set_icon_color)
        qp.drawPixmap(
            (rect.width() - icon.width()) / 2, 
            (rect.height() - icon.height()) / 2,
            icon
        )        
        painter.end()

    # REPAINT BTN
    # Reaload/Repaint BTN
    def repaint_btn(self, event):
        if event == QEvent.Enter:            
            self.repaint()
        if event == QEvent.Leave:            
            self.repaint()
        if event == QEvent.MouseButtonPress:
            self.repaint()
        if event == QEvent.MouseButtonRelease:            
            self.repaint()

    # CHANGE STYLES
    # Functions with custom styles
    def change_style(self, event):
        if event == QEvent.Enter:
            self._set_color = self.color_hover
            self.repaint_btn(event)          
        elif event == QEvent.Leave:
            self._set_color = self.color_default
            self.repaint_btn(event) 
        elif event == QEvent.MouseButtonPress:            
            self._set_color = self.color_pressed
            self._set_icon_color = self.icon_color_pressed
            self.repaint_btn(event) 
        elif event == QEvent.MouseButtonRelease:
            self._set_color = self.color_hover
            self._set_icon_color = self.icon_color
            self.repaint_btn(event) 

    # MOVE TOOLTIP
    def move_tooltip(self):
        # GET MAIN WINDOW PARENT
        gp = self.mapToGlobal(QPoint(0, 0))

        # SET WIDGET TO GET POSTION
        # Return absolute position of widget inside app
        pos = self.parent.mapFromGlobal(gp)

        # FORMAT POSITION
        # Adjust tooltip position with offset
        pos_x = pos.x() + self.width + 12
        pos_y = pos.y() + (self.width - self.tooltip.height()) // 2

        # SET POSITION TO WIDGET
        # Move tooltip position
        self.tooltip.move(pos_x, pos_y)      

    # MOUSE OVER
    # Event triggered when the mouse is over the BTN
    def enterEvent(self, event):
        self.change_style(QEvent.Enter)
        self.move_tooltip()
        self.tooltip.show()

    # MOUSE LEAVE
    # Event fired when the mouse leaves the BTN
    def leaveEvent(self, event):
        self.change_style(QEvent.Leave)
        self.move_tooltip()
        self.tooltip.hide()

    # MOUSE PRESS
    # Event triggered when the left button is pressed
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.change_style(QEvent.MouseButtonPress)
            # EMIT SIGNAL
            self.clicked.emit()
            # SET FOCUS
            self.setFocus()

    # MOUSE RELEASED
    # Event triggered after the mouse button is released
    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.change_style(QEvent.MouseButtonRelease)
            # EMIT SIGNAL
            self.released.emit()

class _ToolTip(QLabel):
    def __init__(self, parent, tooltip):
        QLabel.__init__(self)

        # LABEL SETUP
        self.setObjectName(u"label_tooltip")
        self.setStyleSheet(style_tooltip)
        self.setMinimumHeight(36)
        self.setParent(parent)
        self.setText(tooltip)
        self.adjustSize()

        # SET DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(15)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 160))
        self.setGraphicsEffect(self.shadow)

        # SET OPACITY
        self.opacity = QGraphicsOpacityEffect(self)
        self.opacity.setOpacity(0.85)
        self.setGraphicsEffect(self.opacity)

    



import subprocess
import time
from time import sleep
import os
import pandas as pd
from pylab import np, plt, mpl

class acm_designer(object):
    def __init__(self, work_dir):
        self.work_dir = work_dir # os.path.dirname(os.path.realpath(__file__))

        # optimization related 
        self.bounds_denorm = [  [100, 1000], # BW-current
                                [1.1, 5] ] # delta
        self.counter_fitness_called = 0
        self.counter_fitness_return = 0

        # # simulation-data-aquisition related
        # CL_TS = CL_TS

    def evaluate_design(self, x_denorm):

        with open('_eval/ACMConfig.h', 'r') as f:
            new_line = []
            for line in f.readlines():
                if '#define CURRENT_LOOP_BANDWIDTH' in line:
                    new_line.append(f'#define CURRENT_LOOP_BANDWIDTH (2*M_PI*{x_denorm[0]:g})\n') # 15
                elif '#define DELTA_THE_DAMPING_FACTOR' in line:
                    new_line.append(f'#define DELTA_THE_DAMPING_FACTOR ({x_denorm[1]:g})\n') # 0.08
                else:
                    new_line.append(line)
        with open('_eval/ACMConfig.h', 'w') as f:
            f.writelines(new_line)

        if os.path.exists(self.work_dir+'/_eval/main.exe'):
            os.remove(self.work_dir+'/_eval/main.exe')
        subprocess.run( ["gcc", "_eval/main.c", "_eval/controller.c", "_eval/observer.c", "-o", "_eval/main"] )
        while not os.path.exists(self.work_dir+'/_eval/main.exe'):
            time.sleep(0.1)
            print('sleep for main.exe')

        # subprocess.run( [self.work_dir+'/_eval/main.exe'] )
        os.system('cd _eval && main')
        while not os.path.exists(self.work_dir+'/_eval/pi_opti.dat'):
            time.sleep(0.1)
            print('sleep for .dat')

        speed_profile = np.loadtxt('_eval/pi_opti.dat', skiprows=1)
        os.remove(self.work_dir+'/_eval/pi_opti.dat')

        TS = 0.000025 # ACMConfig.h
        print( max(speed_profile), 'rpm' )
        print( min(speed_profile, key=lambda x:abs(x-90)), 'rpm' ) # [rpm]
        print( np.abs(speed_profile-90).argmin() * TS * 1000, 'ms' ) # [ms]

        rise_time = np.abs(speed_profile-90).argmin() * TS * 1000

        over_shoot = max(speed_profile)
        if over_shoot<=100:
            over_shoot = 100

        # modify this function to call matlab and eMach to evaluate the design with free variables as in x_denorm
        return over_shoot, rise_time

    def update_ACMConfig_evaluate_PI_coefficients(self, currentPI, speedPI, 
            上位机电流PI, 上位机速度PI, 
            max_freq=None, init_freq=2,
            motor_dict=dict(), 
            SWEEP_FREQ_C2V=False,
            SWEEP_FREQ_C2C=False):

        self.motor_dict = motor_dict
        CL_TS = self.motor_dict['CL_TS']

        if max_freq is not None:
            freq_step_size = 1
            endTime = 0
            for current_freq in range(init_freq, max_freq+1, freq_step_size):
                endTime += 1.0/current_freq # 1.0 Duration for each frequency
            NUMBER_OF_STEPS_CL_TS = endTime/CL_TS + 10000
        else:
            endTime = motor_dict['EndTime']
            NUMBER_OF_STEPS_CL_TS = endTime/CL_TS

        if SWEEP_FREQ_C2C:
            # self.data_fname = f"../dat/{data_fname_prefix}-{上位机电流PI[0]:.0f}-{上位机电流PI[1]:.0f}-{上位机速度PI[0]:.0f}-{上位机速度PI[1]:.0f}@{max_freq:.0f}Hz.dat"
            self.data_fname = f"../dat_Closed/{motor_dict['data_fname_prefix']}-@{max_freq:.0f}Hz-{上位机电流PI[0]:.0f}-{上位机电流PI[1]:.0f}-{上位机速度PI[0]:.0f}-{上位机速度PI[1]:.0f}.dat"
        elif SWEEP_FREQ_C2V:
            self.data_fname = f"../dat_Open/{motor_dict['data_fname_prefix']}-@{max_freq:.0f}Hz-{上位机电流PI[0]:.0f}-{上位机电流PI[1]:.0f}-{上位机速度PI[0]:.0f}-{上位机速度PI[1]:.0f}.dat"
        else:
            self.data_fname = f"../dat/{motor_dict['data_fname_prefix']}-{上位机电流PI[0]:.0f}-{上位机电流PI[1]:.0f}-{上位机速度PI[0]:.0f}-{上位机速度PI[1]:.0f}.dat"

        with open(self.work_dir+'/c/ACMConfig.h', 'r') as f:
            new_line = []
            for line in f.readlines():
                if   '#define NUMBER_OF_STEPS' in line: new_line.append(f'#define NUMBER_OF_STEPS {NUMBER_OF_STEPS_CL_TS:.0f}\n')
                elif '#define CURRENT_KP ' in line: new_line.append(f'#define CURRENT_KP ({currentPI[0]:g})\n')
                elif '#define CURRENT_KI ' in line: new_line.append(f'#define CURRENT_KI ({currentPI[1]:g})\n')
                elif '#define SPEED_KP '   in line: new_line.append(f'#define SPEED_KP ({speedPI[0]:g})\n')
                elif '#define SPEED_KI '   in line: new_line.append(f'#define SPEED_KI ({speedPI[1]:g})\n')
                elif '#define CL_TS '      in line: new_line.append(f'#define CL_TS ({CL_TS:g})\n')
                elif '#define CL_TS_INVERSE'        in line: new_line.append(f'#define CL_TS_INVERSE ({1.0/CL_TS:g})\n')
                elif '#define SPEED_LOOP_CEILING'   in line: new_line.append(f'#define SPEED_LOOP_CEILING (4)\n')
                elif '#define SWEEP_FREQ_MAX_FREQ'  in line: new_line.append(f'#define SWEEP_FREQ_MAX_FREQ {max_freq:.0f}\n')
                elif '#define SWEEP_FREQ_INIT_FREQ' in line: new_line.append(f'#define SWEEP_FREQ_INIT_FREQ {init_freq:.0f}\n')
                elif '#define SWEEP_FREQ_C2V'       in line: new_line.append(f'#define SWEEP_FREQ_C2V {"TRUE" if SWEEP_FREQ_C2V else "FALSE"}\n')
                elif '#define SWEEP_FREQ_C2C'       in line: new_line.append(f'#define SWEEP_FREQ_C2C {"TRUE" if SWEEP_FREQ_C2C else "FALSE"}\n')
                elif '#define PMSM_NUMBER_OF_POLE_PAIRS'          in line: new_line.append(f'#define PMSM_NUMBER_OF_POLE_PAIRS          {motor_dict["n_pp"]}\n')
                elif '#define PMSM_RESISTANCE'                    in line: new_line.append(f'#define PMSM_RESISTANCE                    {motor_dict["Rs"]}\n')
                elif '#define PMSM_D_AXIS_INDUCTANCE'             in line: new_line.append(f'#define PMSM_D_AXIS_INDUCTANCE             {motor_dict["Ld"]}\n')
                elif '#define PMSM_Q_AXIS_INDUCTANCE'             in line: new_line.append(f'#define PMSM_Q_AXIS_INDUCTANCE             {motor_dict["Lq"]}\n')
                elif '#define PMSM_PERMANENT_MAGNET_FLUX_LINKAGE' in line: new_line.append(f'#define PMSM_PERMANENT_MAGNET_FLUX_LINKAGE {motor_dict["KE"]}\n')
                elif '#define PMSM_SHAFT_INERTIA'                 in line: new_line.append(f'#define PMSM_SHAFT_INERTIA                 {motor_dict["J_s"]}\n')
                elif '#define PMSM_RATED_CURRENT_RMS'             in line: new_line.append(f'#define PMSM_RATED_CURRENT_RMS             {motor_dict["IN"]}\n')
                elif '#define PMSM_RATED_POWER_WATT'              in line: new_line.append(f'#define PMSM_RATED_POWER_WATT              {motor_dict["PW"]}\n')
                elif '#define PMSM_RATED_SPEED_RPM'               in line: new_line.append(f'#define PMSM_RATED_SPEED_RPM               {motor_dict["RPM"]}\n')
                elif '#define LOAD_INERTIA'                       in line: new_line.append(f'#define LOAD_INERTIA                       {motor_dict["JLoadRatio"]}\n')
                elif '#define LOAD_TORQUE'                        in line: new_line.append(f'#define LOAD_TORQUE                        {motor_dict["Tload"]}\n')
                elif '#define VISCOUS_COEFF'                      in line: new_line.append(f'#define VISCOUS_COEFF                      {motor_dict["ViscousCoeff"]}\n')
                elif '#define DATA_FILE_NAME'                     in line: new_line.append(f'#define DATA_FILE_NAME "{self.data_fname}"\n')
                else: new_line.append(line)
        with open(self.work_dir+'/c/ACMConfig.h', 'w') as f:
            f.writelines(new_line)        

    def plot_PI_coefficients(self, key_ref='ACM.rpm_cmd', key_qep='sm.omg_elec*RAD_PER_SEC_2_RPM'):
        # read data and process
        df_info = pd.read_csv(self.work_dir+r"/dat/info.dat", na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
        data_file_name = df_info['DATA_FILE_NAME'].values[0].strip()
        # print(data_file_name)
        df_profiles = pd.read_csv(self.work_dir+'/dat/'+data_file_name, na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])

        print(self.work_dir+'/dat/'+data_file_name)
        print(df_profiles)
        sleep(3) # wait 3 sec, or else ,analyzer will read in a incomplete .dat file for df_profiles.

        no_samples = df_profiles.shape[0]
        no_traces  = df_profiles.shape[1]
        print(df_info, 'Simulated time: %g s.'%(no_samples * df_info['CL_TS'].values[0] * df_info['DOWN_SAMPLE'].values[0]), 'Key list:', sep='\n')
        for key in df_profiles.keys():
            print('\t', key)

        t = np.arange(1, no_samples+1) * df_info['DOWN_SAMPLE'].values[0] * df_info['CL_TS'].values[0]

        begin = 0
        end = -1
        if 'rpm' not in key_ref:
            plt.figure(11)
            plt.subplot(311)
            plt.title('[Simulated] Current and Speed Profiles')
            plt.plot(t[begin:end], df_profiles[key_ref].values[begin:end], label=key_ref)
            plt.ylabel(key_ref)
            plt.xlabel('Time [s]')
            plt.legend(loc='upper left')

            plt.subplot(312)
            plt.plot(t[begin:end], df_profiles[key_qep].values[begin:end], '--', label=key_qep, lw=0.9)
            plt.ylabel(key_qep)
            plt.xlabel('Time [s]')
            plt.legend(loc='upper left')

            if 'id' not in key_qep:
                plt.subplot(313)
                plt.plot(t[begin:end], df_profiles[key_qep].values[begin:end]/60*2*np.pi*self.motor_dict["n_pp"], '--', label=key_qep, lw=0.9)
                plt.ylabel('Speed [elec.rad/s]')
                plt.xlabel('Time [s]')
                plt.legend(loc='upper left')
            
        else:
            plt.figure(11)
            plt.title('[Simulated] Speed Profiles')
            plt.plot(t[begin:end], df_profiles[key_ref].values[begin:end], label=key_ref)
            plt.plot(t[begin:end], df_profiles[key_qep].values[begin:end], '--', label=key_qep, lw=0.9)
            plt.ylabel('Speed [rpm]')
            plt.xlabel('Time [s]')
            plt.legend(loc='upper left')
        # plt.show()

        return data_file_name

    def compile_c_and_run(self):
        os.system(f"cd /d {self.work_dir}/c && gmake main && start cmd /c main")
        return self.data_fname


if __name__ == '__main__':

    motor_dict = dict()
    motor_dict['n_pp'] = 4
    motor_dict['Rs']   =  0.152
    motor_dict['Ld']   =  0.000466
    motor_dict['Lq']   =  0.000466
    motor_dict['KE']   =  0.023331
    motor_dict['J_s']  = 1.6000000000000003e-05
    motor_dict['IN']   =  12.8
    motor_dict['PW']   =  400
    motor_dict['RPM']  = 3000 



    CURRENT_KP = 3.22076
    CURRENT_KI = 326.18
    SPEED_KP = 0.0303833
    SPEED_KI = 163.586
    currentPI = [CURRENT_KP, CURRENT_KI]
    speedPI = [SPEED_KP, SPEED_KI]
    上位机电流PI = [1100, 1000]
    上位机速度PI = [302.228, 344.581]


    data_fname_prefix = 'simulator-as-main'

    max_freq = 233*2
    SWEEP_OPEN_LOOP = False

    max_freq = 2000
    SWEEP_OPEN_LOOP = True

    ad = acm_designer()
    ad.update_ACMConfig_evaluate_PI_coefficients(currentPI, speedPI, 上位机电流PI, 上位机速度PI, 
                                                 max_freq=max_freq, 
                                                 motor_dict=motor_dict,
                                                 SWEEP_OPEN_LOOP=SWEEP_OPEN_LOOP)
    data_fname = ad.compile_c_and_run()
    if SWEEP_OPEN_LOOP == True:
        ad.plot_PI_coefficients(key_ref='iq_ref', key_qep='sm.omg')
    else:
        ad.plot_PI_coefficients()
    plt.show()



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT
# ///////////////////////////////////////////////////////////////
# Packages
# from app.packages.pyside_or_pyqt import *
# from app.packages.widgets import *
# from PySide2.QtWidgets import *
# from PySide2.QtCore import Signal
from app_modules import * # Settings
# GUI
# from app.uis.main_window.ui_main import Ui_MainWindow # MainWindow
# from app.modules.app_settings.settings import *

# GLOBAL VARS
# ///////////////////////////////////////////////////////////////
_is_maximized = False

# APP FUNCTIONS
# ///////////////////////////////////////////////////////////////
class UiFunctions:

    def __init__(self):
        super(UiFunctions, self).__init__()
        # GET WIDGETS FROM "ui_main.py"
        # Load widgets inside App Functions
        # ///////////////////////////////////////////////////////////////
        self.ui = Ui_PyBlackBox()
        self.ui.setupUi(self)

    # SET UI DEFINITIONS
    # Set ui definitions before "self.show()" in main.py
    # ///////////////////////////////////////////////////////////////
    def maximize_restore(self):
        global _is_maximized
        
        # CHANGE UI AND RESIZE GRIP
        def change_ui():
            if not _is_maximized:
                self.resize(self.width()+1, self.height()+1)
                self.ui.margins_app.setContentsMargins(10, 10, 10, 10)
                self.ui.maximize_restore_app_btn.setToolTip("Restore")
                self.ui.maximize_restore_app_btn.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_maximize.svg);")
                self.ui.bg_app.setStyleSheet("#bg_app { border-radius: 10px; border: 2px solid rgb(30, 32, 33); }")
                self.left_grip.show()
                self.right_grip.show()
                self.top_grip.show()
                self.bottom_grip.show()
                
            else:
                self.ui.margins_app.setContentsMargins(0, 0, 0, 0)
                self.ui.maximize_restore_app_btn.setToolTip("Restore")
                self.ui.maximize_restore_app_btn.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_restore.svg);")
                self.ui.bg_app.setStyleSheet("#bg_app { border-radius: 0px; border: none; }")
                self.left_grip.hide()
                self.right_grip.hide()
                self.top_grip.hide()
                self.bottom_grip.hide()

        # CHECK EVENT
        if self.isMaximized():
            _is_maximized = False
            self.showNormal()
            change_ui()
        else:
            _is_maximized = True
            self.showMaximized()
            change_ui()

    # START CHAT SELECTION
    # ///////////////////////////////////////////////////////////////
    def select_chat_message(self, widget):
        for w in self.ui.messages_frame.findChildren(QWidget):
            if w.objectName() == widget:
                w.set_active(True)

    # RESET CHAT SELECTION
    # ///////////////////////////////////////////////////////////////
    def deselect_chat_message(self, widget):
        for w in self.ui.messages_frame.findChildren(QWidget):
            if w.objectName() != widget:
                if hasattr(w, 'set_active'):
                    w.set_active(False)

    # SET UI DEFINITIONS
    # Set ui definitions before "self.show()" in main.py
    # ///////////////////////////////////////////////////////////////
    def set_ui_definitions(self):

        # GET SETTINGS FROM JSON DESERIALIZED
        settings = Settings()
        self.settings = settings.items

        # REMOVE TITLE BAR
        # ///////////////////////////////////////////////////////////////
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        # MOVE WINDOW / MAXIMIZE / RESTORE
        def moveWindow(event):
            # IF MAXIMIZED CHANGE TO NORMAL
            if self.isMaximized():
                UiFunctions.maximize_restore(self)
                curso_x = self.pos().x()
                curso_y = event.globalPos().y() - QCursor.pos().y()
                self.move(curso_x, curso_y)
            # MOVE WINDOW
            if event.buttons() == Qt.LeftButton:
                self.move(self.pos() + event.globalPos() - self.dragPos)
                self.dragPos = event.globalPos()
                event.accept()
        self.ui.logo_top.mouseMoveEvent = moveWindow
        self.ui.title_bar.mouseMoveEvent = moveWindow

        # DOUBLE CLICK MAXIMIZE / RESTORE
        def maximize_restore(event):
            if event.type() == QEvent.MouseButtonDblClick:
                UiFunctions.maximize_restore(self)
        self.ui.title_bar.mouseDoubleClickEvent = maximize_restore

        # TOP BTNS
        self.ui.minimize_app_btn.clicked.connect(lambda: self.showMinimized())        
        self.ui.maximize_restore_app_btn.clicked.connect(lambda: UiFunctions.maximize_restore(self))
        self.ui.close_app_btn.clicked.connect(lambda: self.close())
        

        
        # DEFAULT PARAMETERS
        self.setWindowTitle(self.settings["app_name"])
        self.resize(self.settings["startup_size"][0], self.settings["startup_size"][1])
        self.setMinimumSize(self.settings["minimum_size"][0], self.settings["minimum_size"][1])

        # APPLY DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect()
        self.shadow.setBlurRadius(25)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 80))
        self.ui.stylesheet.setGraphicsEffect(self.shadow)

        # CUSTOM GRIPS
        # Create grips to resize window
        self.left_grip = CustomGrip(self, Qt.LeftEdge, True)
        self.right_grip = CustomGrip(self, Qt.RightEdge, True)
        self.top_grip = CustomGrip(self, Qt.TopEdge, True)
        self.bottom_grip = CustomGrip(self, Qt.BottomEdge, True)

    # RESIZE GRIPS
    # This function should be called whenever "MainWindow/main.py" has its window resized.
    # ///////////////////////////////////////////////////////////////
    def resize_grips(self):
        self.left_grip.setGeometry(0, 10, 10, self.height())
        self.right_grip.setGeometry(self.width() - 10, 10, 10, self.height())
        self.top_grip.setGeometry(0, 0, self.width(), 10)
        self.bottom_grip.setGeometry(0, self.height() - 10, self.width(), 10)
